@@ -1,7 +1,8 @@
 from pysb.core import *
+from pysb.annotation import Annotation
 
 __all__ = ['Observable', 'Initial', 'MatchOnce', 'Model', 'Monomer',
-           'Parameter', 'Compartment', 'Rule', 'ANY', 'WILD']
+           'Parameter', 'Compartment', 'Rule', 'ANY', 'WILD', 'Annotation']
 
 try:
     import reinteract         # fails if reinteract not installed
