@@ -156,8 +156,13 @@ def format_site_condition(site, state):
             state = (state[0], '?')
         state_code = '~%s!%s' % state
     # one or more unspecified bonds
-    elif state == pysb.ANY:
+    elif state is pysb.ANY:
         state_code = '!+'
+    # anything at all (usually you can leverage don't-care-don't-write, but in
+    # some cases such as when a rule explicitly sets the state of site A but
+    # conditions on site B, site A on the reactant side must be set to WILD)
+    elif state is pysb.WILD:
+        state_code = '!?'
     else:
         raise Exception("BNG generator has encountered an unknown element in a rule pattern site condition.")
     return '%s%s' % (site, state_code)
